import re


def parse_results(soup):
    """
    Parse search results from the advanced search page.

    Parameters
    ----------
    content : BeautifulSoup
        BeautifulSoup object of the search page content.

    Returns
    -------
    results : list of dicts
        List of search results.
        ::

            [
                {
                    'name': 'Series Name',
                    'id': 'Series ID',
                    'rating': 'Average Rating',
                    'summary': 'Short summary',
                    'thumbnail': 'Thumbnail link' or None,
                    'year': 'Year manga started releasing',
                    'is_adult': True or False
                    # note! mangaupdates categorizes whole genres like 'Yuri' as adult
                    # so use this at your own discretion :)
                    # if this is true, thumbnail will always be None.
                }
            ]
    """
    results = []
    for series in soup.find_all("div", attrs={"class": "col-12 col-lg-6 p-3 text"}):
        box = series.div.find_all("div", recursive=False)

        info = box[1].div.find_all("div", recursive=False)

        ratings_text = re.search(
            r"(?P<year>\d{4})*(?: - )*(?:(?P<rating>\d+\.(\d+)) \/ 10\.0)*",
            info[3].get_text(),
        )

        thumbnail = box[0].find("img")
        is_adult = False
        if not thumbnail:
            is_adult = bool(box[0].find(lambda tag: tag.get_text() == "AdultContent"))
        else:
            thumbnail = thumbnail["src"]
            # when a manga is considered to contain 'Adult Content', mangaupdates replaces the thumbnail with a divsoup that renders to a centered 'Adult Content' on a white background.

        results.append(
            {
                "name": info[0].a.u.b.get_text(),
                "id": info[0]
                .a["href"]
                .replace("https://www.mangaupdates.com/series.html?id=", ""),
                "genres": info[1].a["title"].split(", "),
                "summary": info[2].get_text(),
                "year": ratings_text.group("year"),
                "rating": ratings_text.group("rating"),
                "thumbnail": thumbnail,
                "is_adult": is_adult,
            }
        )
    return results



"""
List of all MangaUpdates genres.
    * 'Action'
    * 'Adult'
    * 'Adventure'
    * 'Comedy'
    * 'Doujinshi'
    * 'Drama'
    * 'Ecchi'
    * 'Fantasy'
    * 'Gender Bender'
    * 'Harem'
    * 'Hentai'
    * 'Historical'
    * 'Horror'
    * 'Josei'
    * 'Lolicon'
    * 'Martial Arts'
    * 'Mature'
    * 'Mecha'
    * 'Mystery'
    * 'Psychological'
    * 'Romance'
    * 'School Life'
    * 'Sci-fi'
    * 'Seinen'
    * 'Shotacon'
    * 'Shoujo'
    * 'Shoujo Ai'
    * 'Shounen'
    * 'Shounen Ai'
    * 'Slice of Life'
    * 'Smut'
    * 'Sports'
    * 'Supernatural'
    * 'Tragedy'
    * 'Yaoi'
    * 'Yuri'
"""
genres = [
    "Action",
    "Adult",
    "Adventure",
    "Comedy",
    "Doujinshi",
    "Drama",
    "Ecchi",
    "Fantasy",
    "Gender Bender",
    "Harem",
    "Hentai",
    "Historical",
    "Horror",
    "Josei",
    "Lolicon",
    "Martial Arts",
    "Mature",
    "Mecha",
    "Mystery",
    "Psychological",
    "Romance",
    "School Life",
    "Sci-fi",
    "Seinen",
    "Shotacon",
    "Shoujo",
    "Shoujo Ai",
    "Shounen",
    "Shounen Ai",
    "Slice of Life",
    "Smut",
    "Sports",
    "Supernatural",
    "Tragedy",
    "Yaoi",
    "Yuri",
]



def parse_releases(list):
    """
    Parse releases from a MangaUpdate's search results page.

    Parameters
    ----------
    list : BeautifulSoup
        BeautifulSoup object of the releases section of the search page.

    Returns
    -------
    releases : list of dicts
        List of recent releases found by the search.
        ::

            [
                {
                    'id': 'Series Id',
                    'name': 'Series name',
                    'chp': 'chapter number',
                    'vol': 'number' or None,
                    'date': '02/21/21', # Date in month/day/year
                    'group': {
                        'name': 'Scanlation Group',
                        'id': 'Scanlation Group Id'
                    }
                }
            ]
    """
    releases = list.find_all("div", class_="text")[:-1]
    results = []

    for i in range(0, len(releases), 5):
        release = {}

        release["date"] = str(releases[i].string)

        series_link = releases[i + 1]
        if series_link.a is None:
            release["name"] = str(series_link.string)
        else:
            release["name"] = str(series_link.a.string)
            release["id"] = series_link.a["href"].replace(
                "https://www.mangaupdates.com/series.html?id=", ""
            )

        vol = releases[i + 2].get_text()
        release["vol"] = vol if vol else None
        release["chp"] = str(releases[i + 3].string)
        release["group"] = {
            "name": releases[i + 4].get_text(),
            "id": releases[i + 4]
            .a["href"]
            .replace("https://www.mangaupdates.com/groups.html?id=", ""),
        }

        results.append(release)

    return results


def parse_series(list):
    """
    Parse series from a MangaUpdate's search results page.

    Parameters
    ----------
    list : BeautifulSoup
        BeautifulSoup object of the series section of the search results page.

    Returns
    -------
    series : list of dicts
        List of manga found by the search.
        ::

            [
                {
                    'name': 'Manga Name',
                    'id': 'Manga Id',
                    'rating': '7.80' # average (?) rating of manga on mangaupdates,
                    'year': '2018', # year manga started releasing
                    'genres': ['Drama', 'Shoujo Ai', 'S...'] # genres this series is a part of
                    # (note: sometimes, the last item of this will be cut off with ...)
                }
            ]
    """

    series = list.find_all("div", class_="text")[:-1]
    results = []

    for i in range(0, len(series), 4):
        manga = {}

        series_link = series[i]
        if series_link.a is None:
            manga["name"] = str(series_link.string)
        else:
            manga["name"] = str(series_link.a.string)
            manga["id"] = series_link.a["href"].replace(
                "https://www.mangaupdates.com/series.html?id=", ""
            )

        manga["genres"] = [
            genre.strip() for genre in series[i + 1].get_text().split(",")
        ]
        manga["year"] = str(series[i + 2].string)
        manga["rating"] = str(series[i + 3].string)

        results.append(manga)

    return results


def parse_scanlators(list):
    """
    Parse scanlation groups from a MangaUpdate's search results page.

    Parameters
    ----------
    list : BeautifulSoup
        BeautifulSoup object of the scanlators section of the search results page.

    Returns
    -------
    scanlators : list of dicts
        List of scanlation groups found by the search.
        ::

            [
                {
                    'name': 'Scanlation Group Name',
                    'id': 'Scanlation Group ID',
                    'active': True or False,
                    'contact': [
                        'contact link',
                        ...
                    ]
                }
            ]
    """
    scanlators = list.find_all("div", class_="text")[:-1]
    results = []

    for i in range(0, len(scanlators), 3):
        group = {}

        group_link = scanlators[i]
        if group_link.a is None:
            group["name"] = str(group_link.string)
        else:
            group["name"] = str(group_link.a.string)
            group["id"] = group_link.a["href"].replace(
                "https://www.mangaupdates.com/groups.html?id=", ""
            )

        group["active"] = True if scanlators[i + 1].get_text() == "Yes" else False

        group_contact = scanlators[i + 2]
        if group_contact.a is None:
            group["contact"] = group_contact.get_text()
        else:
            group["contact"] = [a["href"] for a in group_contact.find_all("a")]

        results.append(group)

    return results


def parse_authors(list):
    """
    Parse authors from a MangaUpdate's search results page.

    Parameters
    ----------
    list : BeautifulSoup
        BeautifulSoup object of the authors section of the search results page.

    Returns
    -------
    series : list of dict
        List of authors found by the search.
        ::

            [
                {
                    'name': 'Author Name',
                    'id': 'ID',
                    'series': int(Number of series author has done),
                    'genres': ['Genre',...] # most numerous genre(s)
                                            # in this author's work
                }
            ]
    """
    authors = list.find_all("div", class_="text")[:-1]
    results = []

    for i in range(0, len(authors), 3):
        author = {}

        author_link = authors[i]
        if author_link.a is None:
            author["name"] = author_link.get_text()
        else:
            author["name"] = author_link.get_text()
            author["id"] = author_link.a["href"].replace(
                "https://www.mangaupdates.com/authors.html?id=", ""
            )

        author["series"] = int(authors[i + 1].get_text())

        author["genres"] = []

        for gen in authors[i + 2].find_all("a"):
            author["genres"].append(gen.get_text())

        results.append(author)

    return results



import markdownify, html2text
from bs4 import Comment
import re


def parse_series(content, description_format="markdown"):
    """
    Parse series info from mangaupdates.

    Parameters
    ----------
    content : BeautifulSoup
        BeautifulSoup object of series page html.
    description_format : str, optional
        Format to transform the description into. can be 'plain', 'raw' or 'markdown'. defaults to 'markdown'.

    Returns
    -------
    series : dict
        Series information.
        ::

            {
                # main info
                'title': 'Series Name',
                'year':
                'type': 'Type of series (manhwa,manhua,manga,et)',
                'status': 'n Volumes (Ongoing or Complete or etc..)'
                'image': 'cover image link',
                'last_updated': 'December 3rd 2020, 5:32pm PST', # last time page was updated
                # authors & artists
                'artists': [
                    {
                        'id': 'Artist's Manga Updates ID',
                        'name': 'Artist's Name'
                    }
                ],
                'authors': [
                    {
                        'id': 'Author's Manga Updates ID',
                        'name': 'Author's Name'
                    }
                ],
                # relations
                'associated_names': [ # often in different languages, so make sure you can handle unicode.
                     'Name one',
                     'Name two'
                ],
                'related_series': [
                    {
                        'id': 'Related Series ID',
                        'name': 'Related Series Name',
                        'relation': 'Relation to current manga'
                    }
                ],
                'anime_chapters': [ # if it doesn't have an anime, list will be empty
                    'Starts at Vol x, Chap y',
                    'Ends at Vol z, Chap w'
                ],
                # description & genre
                'genres': [
                    'Genre',
                    ...
                ],
                'categories': [
                    {
                        'category': 'Category Name',
                        'score': '16 (16,0)'
                    }
                ],
                'description': 'Lorem ipsum dolor sit amet..',
                # publishing info
                'publisher': {
                    'id': 'publisher ID',
                    'name': 'publisher name'
                },
                'serialized': {
                    'link': 'mangaupdates link to where it was serialized',
                    'name': 'name of where it was serialized'
                },
                'licensed': True or False # whether this series was licensed in english,
                'english_publisher': {
                    'id': 'English Publisher ID',
                    'name': 'English Publisher Name + Volume Info'
                },
                # scanlation info
                'completely_scanlated': True or False,
                'latest_releases': [
                    {
                        'chapter': 'chapter number',
                        'volume': 'volume number if present',
                        'date': 'n days ago',
                        'group': {
                            'id': 'Scanlation Group ID',
                            'name': 'Scanlation Group Name'
                        }
                    }
                ],
                'groups': [ # all scanlation groups that published releases for this series
                    {
                        'id': 'Scanlation Group ID',
                        'name': 'Scanlation Group Name'
                    }
                ],
                # recommendations
                'category_recs': [
                    {
                        'id': 'Recommended Series ID',
                        'name': 'Recommended Series Name'
                    }
                ],
                'recs': [
                    {
                        'id': 'Recommended Series ID',
                        'name': 'Recommended Series Name'
                    }
                ],
                # user-related info
                'positions': {
                    'monthly': '779',
                    'monthly_change': '+155',
                    'six_monthly': '1244',
                    'six_monthly_change': '+76',
                    'tri_monthly': '1120',
                    'tri_monthly_change': '-17',
                    'weekly': '431',
                    'weekly_change': '+121',
                    'yearly': '1277',
                    'yearly_change': '-162'
                },
                'average': { # ratings
                    'average': 'average rating',
                    'bayesian': 'bayesian average rating',
                    'votes': 'n votes'
                },
                'reading_lists': {
                    'custom': 'n',
                    'reading': 'n',
                    'unfinished': 'n',
                    'wish': 'n'
                },
                'forum': {
                    'link': 'https://www.mangaupdates.com/topics.php?fid=120202',
                    'status': 'None'
                }
            }


    """
    manga = {}
    manga["title"] = str(content.find("span", class_="releasestitle").get_text())

    cols = content.find_all("div", class_="col-6", recursive=False)
    col_1 = cols[0]
    col_2 = cols[1]

    _parse_col_1(col_1, manga, description_format)
    _parse_col_2(col_2, manga)
    return manga


def _parse_col_1(col, manga, description_format):
    contents = col.find_all("div", class_="sContent", recursive=False)

    desc_tag = contents[0]

    desc_more = desc_tag.find(id="div_desc_more")
    if desc_more:
        desc_more.a.extract()
        desc_html = desc_more
    else:
        desc_html = desc_tag

    if description_format == "markdown":
        manga["description"] = markdownify.markdownify(str(desc_html))
    elif description_format == "raw":
        manga["description"] = str(desc_html)
    elif description_format == "plain":
        text_converter = html2text.HTML2Text()
        text_converter.ignore_links = True
        text_converter.unicode_snob = True
        text_converter.ignore_emphasis = True
        text_converter.ignore_anchors = True
        text_converter.ignore_images = True
        manga["description"] = text_converter.handle(str(desc_html))

    manga["type"] = contents[1].get_text().replace("\n", "")

    manga["related_series"] = []
    for link in contents[2].findAll("a"):
        manga["related_series"].append(
            {
                "name": link.get_text(),
                "id": link.get("href", "").replace("series.html?id=", ""),
                "relation": str(link.nextSibling)
                .strip()
                .replace("(", "")
                .replace(")", ""),
            }
        )

    manga["associated_names"] = [
        markdownify.markdownify(name)
        for name in contents[3]
        .encode_contents()
        .decode("utf-8")
        .replace("\n", "")
        .replace("</br>", "")
        .split("<br>")
        if len(name) > 0 and name != "N/A"
    ]

    if "N/A" not in contents[4].get_text():
        if contents[4].a is None:
            manga["groups"] = [{"name": contents[4].get_text().strip(), "id": None}]
        else:
            manga["groups"] = []
            for group in contents[4].findAll("a", attrs={"title": "Group Info"}):
                manga["groups"].append(
                    {
                        "name": group.get_text(),
                        "id": group["href"].replace(
                            "https://www.mangaupdates.com/groups.html?id=", ""
                        ),
                    }
                )
    else:
        manga["groups"] = []

    manga["latest_releases"] = []
    numbers = contents[5].find_all("i")[:-1]
    groups = contents[5].find_all("a")[:-1]
    dates = contents[5].find_all("span")

    for i in range(0, len(dates)):
        release = {
            "group": {
                "name": groups[i].get_text(),
                "id": str(
                    groups[i]
                    .get("href", "")
                    .replace("https://www.mangaupdates.com/groups.html?id=", "")
                ),
            },
            "date": dates[i]["title"],
        }

        # this is to check if there are volume numbers. its a bad solution, folks!
        if len(numbers) >= len(dates) * 2:
            release["volume"] = numbers[i].get_text()
            release["chapter"] = numbers[i + 1].get_text()
        else:
            release["chapter"] = numbers[i].get_text()

        manga["latest_releases"].append(release)

    manga["status"] = (
        contents[6].get_text(separator="!@#").replace("\n", "").split("!@#")
    )
    if str(contents[7].string).replace("\n", "") == "No":
        manga["completely_scanlated"] = False
    else:
        manga["completely_scanlated"] = True

    manga["anime_chapters"] = (
        None
        if "N/A" in contents[8].get_text()
        else contents[8]
        .encode_contents()
        .decode("utf-8")
        .replace("\n", "")
        .split("<br/>")
    )
    manga["user_reviews"] = contents[9].get_text().replace("\n", "")

    manga["forum"] = {
        "status": contents[10].get_text(),
        "link": "https://www.mangaupdates.com/" + contents[10].a.get("href", ""),
    }

    try:
        average_raw = contents[11].contents
        manga["average"] = {
            "average": str(average_raw[0]).replace("Average:", "").replace(" ", ""),
            "votes": str(average_raw[2]).replace("(", "").replace(")", ""),
            "bayesian": str(average_raw[5]).replace("<b>", "").replace("</b>", ""),
        }
    except:
        manga["average"] = None

    manga["last_updated"] = contents[12].get_text().replace("\n", "")


def _parse_col_2(col, manga):
    contents = col.find_all("div", class_="sContent", recursive=False)

    manga["image"] = contents[0].center.img["src"]

    manga["genres"] = []
    for genre in contents[1].find_all("a")[:-1]:
        manga["genres"].append(genre.u.get_text())

    manga["categories"] = []
    if contents[2].div:
        for cat_raw in contents[2].div.ul.find_all("li"):
            cat = cat_raw.find("a", rel="nofollow")
            manga["categories"].append(
                {
                    "category": cat.get_text(),
                    "score": str(cat["title"]).replace("Score:", ""),
                }
            )

    manga["category_recs"] = []
    for rec in contents[3].find_all("a", recursive=True):
        if "javascript" not in rec.get("href", ""):
            manga["category_recs"].append(
                {
                    "name": rec.get_text(),
                    "id": rec.get("href", "").replace("series.html?id=", ""),
                }
            )

    manga["recs"] = []
    for rec in contents[4].find_all("a", recursive=True):
        if "javascript" not in rec.get("href", ""):
            manga["recs"].append(
                {
                    "name": rec.get_text(),
                    "id": rec.get("href", "").replace("series.html?id=", ""),
                }
            )

    manga["authors"] = []
    for author in contents[5].find_all("a"):
        manga["authors"].append(
            {
                "name": author.get_text(),
                "id": author.get("href", "").replace(
                    "https://www.mangaupdates.com/authors.html?id=", ""
                )
                if contents[5].a
                else "N/A",
            }
        )

    manga["artists"] = []
    for artist in contents[6].find_all("a"):
        manga["artists"].append(
            {
                "name": artist.get_text(),
                "id": artist.get("href", "").replace(
                    "https://www.mangaupdates.com/authors.html?id=", ""
                )
                if contents[6].a
                else "N/A",
            }
        )

    manga["year"] = contents[7].get_text().replace("\n", "")

    manga["publisher"] = {
        "name": contents[8].get_text().strip(),
        "id": contents[8]
        .a.get("href", "")
        .replace("https://www.mangaupdates.com/publishers.html?id=", "")
        if contents[8].a
        else "N/A",
    }

    # TODO: add publisher info
    manga["serialized"] = {
        "name": contents[9].get_text().strip(),
        "link": "https://www.mangaupdates.com/" + contents[9].a.get("href", "")
        if contents[9].a
        else "",
    }

    manga["licensed"] = True if "Yes" in contents[10].get_text() else False

    # TODO: add volume/ongoing info
    manga["english_publisher"] = {
        "name": str(contents[11].get_text().strip()),
        "id": str(
            contents[11]
            .a.get("href", "")
            .replace("https://www.mangaupdates.com/publishers.html?id=", "")
            if contents[11].a
            else "N/A"
        ),
    }

    positions = dict()
    pos_r = str(contents[12]).split('<br/>')
    for title, p in zip(['weekly', 'monthly', 'tri_monthly', 'six_monthly', 'yearly'], pos_r):
        position = re.search('<b>(\d+)</b>', p)
        change = re.search('\(([^()]+)\)', p)

        positions[title] = position.group(1)
        if change is not None:
            positions[title+'_change'] = change.group(1)
        else:
            positions[title+'_change'] = ''

    manga["positions"] = positions

    read_lists = contents[13].find_all("b")
    manga["reading_lists"] = {
        "reading": read_lists[0].get_text().strip(),
        "wish": read_lists[1].get_text().strip(),
        "unfinished": read_lists[2].get_text().strip(),
        "custom": read_lists[3].get_text().strip(),
    }



import requests
from bs4 import BeautifulSoup
import urllib.parse as urlparse
import re
import os
from .parsers import search_parsers, series_parsers, releases_parsers, adv_search_parser


def search(query):
    """
    Performs a name-based search of MangaUpdates.

    Parameters
    ----------
    query : str
        Name to search by.

    Returns
    -------
    dict : dict of {str : list}
        Results of search.

    See Also
    --------
    pymanga.parsers.search_parsers
    """

    r = requests.post("https://mangaupdates.com/search.html", params={"search": query})
    soup = BeautifulSoup(r.text, "html.parser")
    lists = (
        soup.find("div", class_="center-side-bar")
        .find_all("div", class_="row", recursive=False)[1]
        .find("div", id="main_content")
        .find_all("div", class_="row")
    )

    results = {}
    try:
        results["releases"] = search_parsers.parse_releases(lists[0])
    except:
        results["releases"] = []
    try:
        results["series"] = search_parsers.parse_series(lists[1])
    except:
        results["series"] = []
    try:
        results["scanlators"] = search_parsers.parse_scanlators(lists[2])
    except:
        results["scanlators"] = []
    try:
        results["authors"] = search_parsers.parse_authors(lists[3])
    except:
        results["authors"] = []
    return results


# Warning: this can take a long time on general searches with the 'all_pages' option enabled!
def advanced_search(params, all_pages=False, page=1):
    """
    Do an 'advanced search' of MangaUpdates' manga.

    Parameters
    ----------
    params : dict
        Dict of parameters to search by.

        ::

            {
                'orderby': 'rating' or 'title' or 'year', # how to sort search results,
                'category': ['MangaUpdates','Categories/Tags'],
                'genre': ['Genres', 'To', 'Include'], # for a list of possible values
                'exclude_genre': ['Genres', 'To', 'Exclude' # see 'pymanga.genres'

                'licensed': empty or 'yes' or 'no',
                    # empty/not specified -> show all
                    # 'yes' -> show only licensed manga
                    # 'no' -> show only unlicensed manga

                'filter': empty or 'filtering option',
                    # empty/not specified -> show all
                    # 'scanlated' -> only completely scanlated manga
                    # 'completed' -> only completed manga (includes oneshots)
                    # 'oneshots' -> only show oneshots
                    # 'no_oneshots' -> excludes oneshots
                    # 'some_releases' -> only show manga with at least one release
                    # 'no_releases' -> only show manga with no releases

                'type': empty or 'type option',
                    # 'artbook'
                    # 'doujinshi'
                    # 'drama_cd'
                    # 'filipino'
                    # 'indonesian'
                    # 'manga'
                    # 'manhwa'
                    # 'manhua'
                    # 'novel'
                    # 'oel'
                    # 'thai'
                    # 'vietnamese'
                    # 'malaysian'
            }

    all_pages : bool, optional
        Fetch all results pages or not.

        Defaults to False.

        Note: the module automatically sleeps between requests if all_pages is set to true,
        as to not overload MangaUpdates.

        Consider using advanced_search_iter instead.

    page : int, optional
        What page of results to fetch.
        Defaults to 1.

    Returns
    -------
    results : list of dicts
        List of search results.
        ::

            [
                {
                    'name': 'Series Name',
                    'id': 'Series ID',
                    'rating': 'Average Rating',
                    'summary': 'Short summary',
                    'thumbnail': 'Thumbnail link' or None,
                    'year': 'Year manga started releasing',
                    'is_adult': True or False
                    # note! mangaupdates categorizes whole genres like 'Yuri' as adult
                    # so use this at your own discretion :)
                    # if this is true, thumbnail will always be None.
                }
            ]

    See Also
    --------
    pymanga.genres
    pymanga.api.advanced_search_iter
    """
    params["page"] = page
    params["perpage"] = 100

    # do some quality of life processing (turn list args. into properly escaped parameters)
    params["genre"] = "_".join(params.get("genre", []))
    params["exclude_genre"] = "_".join(params.get("exclude_genre", []))
    params["category"] = "_".join(params.get("category", []))

    if "name" in params:
        params["search"] = params["name"]

    payload = urlparse.urlencode(params, safe="+")

    r = requests.get("https://mangaupdates.com/series.html", params=payload)
    soup = BeautifulSoup(r.text, "html.parser")
    # find page count. i know regex is a crime but.

    pages = re.search(r"Pages \((\d+)\)", r.text)
    if pages:
        pages = int(pages.group(1))
    else:
        pages = 1

    results = adv_search_parser.parse_results(soup)

    # if all pages is set, request every page and append their results to the return value

    if all_pages and pages > 1:
        for p in range(page + 1, pages + 1):
            r, _ = advanced_search(params, page=p)
            results = results + r

        os.sleep(1)  # be polite!

    return (results, pages)


# advanced search, but with a generator!
def advanced_search_iter(params):
    """
    Same as pymanga.api.advanced_search, but as a generator.

    Requests new pages from mangaupdates only as needed.

    See Also
    --------
    pymanga.api.advanced_search
    """
    first_batch, total_pages = advanced_search(params)
    yield from first_batch

    for p in range(2, total_pages + 1):
        r, _ = advanced_search(params, page=p)
        yield from r


def series(id, description_format="markdown"):
    """
    Get series info from mangaupdates.

    Parameters
    ----------
    id : str
        Series id.

    description_format : str, optional
        Format to transform the description into. can be 'plain', 'raw' or 'markdown'. defaults to 'markdown'.

    Returns
    -------
    series : dict
        Series information.
        ::

            {
                # main info
                'title': 'Series Name',
                'year':
                'type': 'Type of series (manhwa,manhua,manga,et)',
                'status': 'n Volumes (Ongoing or Complete or etc..)'
                'image': 'cover image link',
                'last_updated': 'December 3rd 2020, 5:32pm PST', # last time page was updated
                # authors & artists
                'artists': [
                    {
                        'id': 'Artist's Manga Updates ID',
                        'name': 'Artist's Name'
                    }
                ],
                'authors': [
                    {
                        'id': 'Author's Manga Updates ID',
                        'name': 'Author's Name'
                    }
                ],
                # relations
                'associated_names': [ # often in different languages, so make sure you can handle unicode.
                     'Name one',
                     'Name two'
                ],
                'related_series': [
                    {
                        'id': 'Related Series ID',
                        'name': 'Related Series Name',
                        'relation': 'Relation to current manga'
                    }
                ],
                'anime_chapters': [ # if it doesn't have an anime, list will be empty
                    'Starts at Vol x, Chap y',
                    'Ends at Vol z, Chap w'
                ],
                # description & genre
                'genres': [
                    'Genre',
                    ...
                ],
                'categories': [
                    {
                        'category': 'Category Name',
                        'score': '16 (16,0)'
                    }
                ],
                'description': 'Lorem ipsum dolor sit amet..',
                # publishing info
                'publisher': {
                    'id': 'publisher ID',
                    'name': 'publisher name'
                },
                'serialized': {
                    'link': 'mangaupdates link to where it was serialized',
                    'name': 'name of where it was serialized'
                },
                'licensed': True or False # whether this series was licensed in english,
                'english_publisher': {
                    'id': 'English Publisher ID',
                    'name': 'English Publisher Name + Volume Info'
                },
                # scanlation info
                'completely_scanlated': True or False,
                'latest_releases': [
                    {
                        'chapter': 'chapter number',
                        'volume': 'volume number if present',
                        'date': 'n days ago',
                        'group': {
                            'id': 'Scanlation Group ID',
                            'name': 'Scanlation Group Name'
                        }
                    }
                ],
                'groups': [ # all scanlation groups that published releases for this series
                    {
                        'id': 'Scanlation Group ID',
                        'name': 'Scanlation Group Name'
                    }
                ],
                # recommendations
                'category_recs': [
                    {
                        'id': 'Recommended Series ID',
                        'name': 'Recommended Series Name'
                    }
                ],
                'recs': [
                    {
                        'id': 'Recommended Series ID',
                        'name': 'Recommended Series Name'
                    }
                ],
                # user-related info
                'positions': {
                    'monthly': '779',
                    'monthly_change': '+155',
                    'six_monthly': '1244',
                    'six_monthly_change': '+76',
                    'tri_monthly': '1120',
                    'tri_monthly_change': '-17',
                    'weekly': '431',
                    'weekly_change': '+121',
                    'yearly': '1277',
                    'yearly_change': '-162'
                },
                'average': { # ratings
                    'average': 'average rating',
                    'bayesian': 'bayesian average rating',
                    'votes': 'n votes'
                },
                'reading_lists': {
                    'custom': 'n',
                    'reading': 'n',
                    'unfinished': 'n',
                    'wish': 'n'
                },
                'forum': {
                    'link': 'https://www.mangaupdates.com/topics.php?fid=120202',
                    'status': 'None'
                }
            }

    """
    r = requests.get("https://mangaupdates.com/series.html", params={"id": id})
    soup = BeautifulSoup(r.text, "html.parser")
    content = (
        soup.find("div", class_="center-side-bar")
        .find_all("div", class_="row", recursive=False)[1]
        .find("div", id="main_content")
        .find("div", class_="p-2", recursive=False)
        .find("div", class_="row", recursive=False)
    )
    return series_parsers.parse_series(content, description_format=description_format)


def releases(id):
    """
    Get latest releases of a manga

    Parameters
    ----------
    id : str
        Manga ID.

    Returns
    -------
    releases : list of dicts
        List of latest releases of a manga.
        List is ordered latest-to-oldest
        ::

            [
                {
                    'chapter': 'chapter number',
                    'vol': 'volume number' or None,
                    'series': {
                        'name': 'Manga Name',
                        'id': 'Manga ID'
                    },
                    'group': {
                        'name': 'Scanlation Group Name',
                        'id': 'Scanlation Group ID'
                    }
                }
            ]
    """
    r = requests.post(
        "https://www.mangaupdates.com/releases.html",
        params={"stype": "series", "search": id, "page": 1},
        data={"perpage": 100},
    )
    soup = BeautifulSoup(r.text, "html.parser")
    content = (
        soup.find("div", class_="center-side-bar")
        .find_all("div", class_="row", recursive=False)[1]
        .find("div", id="main_content")
        .find("div", class_="p-2", recursive=False)
        .find("div", class_="row", recursive=False)
    )
    releases = []
    try:
        releases = releases_parsers.parse_releases(content)
    except:
        releases = []
    return releases



def parse_releases(content):
    """
    Parse latest releases of a manga

    Parameters
    ----------
    content : BeautifulSoup
        BeautifulSoup object of the releases page content.

    Returns
    -------
    releases : list of dicts
        List of latest releases of a manga.
        List is ordered latest-to-oldest
        ::

            [
                {
                    'chapter': 'chapter number',
                    'vol': 'volume number' or None,
                    'series': {
                        'name': 'Manga Name',
                        'id': 'Manga ID'
                    },
                    'group': {
                        'name': 'Scanlation Group Name',
                        'id': 'Scanlation Group ID'
                    }
                }
            ]
    """
    releases = content.find_all("div", class_="text", recursive=False)[:-1]
    results = []

    for i in range(0, len(releases), 5):
        release = {}

        release["series"] = {
            "name": releases[i + 1].get_text(),
            "id": releases[i + 1]
            .a["href"]
            .replace("https://www.mangaupdates.com/series.html?id=", ""),
        }

        vol = releases[i + 2].get_text()
        release["vol"] = vol if vol else None
        release["chapter"] = releases[i + 3].get_text()
        release["group"] = {
            "name": releases[i + 4].get_text(),
            "id": releases[i + 4]
            .a["href"]
            .replace("https://www.mangaupdates.com/groups.html?id=", ""),
        }
        results.append(release)

    return results



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py-manga",
    version="0.1.8",
    author="Allie",
    author_email="allie@cat-girl.gay",
    description="A library to get information from mangaupdates.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/alisw/py-manga",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'beautifulsoup4',
        'requests',
        'markdownify',
        'html2text'
    ],
    python_requires='>=3.4'
)
